################################################################################
#  This R script creates spatial grid and neighbor information, runs spatial   #
#  CJS model in WINBUGS, and outputs results and gridded mean parameter        #
#  estimates for the wood thrush example presented in Saracco, Royle, DeSante, #
#  and Gardner:                                                                #
#  "Modeling Spatial Variation in Avian Survival and Residency Probabilities"  #
#                                                                              #
#  Original model code provided by J.A. Royle and B. Gardner.                  #
#  Last edited 4 December 2009 by J. Saracco                                   #
################################################################################

#load libraries 
library(spdep)         #requires 'Matrix', 'spam', and 'tripack' packages
library(spatstat)      #requires 'gpclib' and 'deldir' packages
library(maptools)
library(R2WinBUGS)   #requires 'coda' package
library(boot)

#set working directory
setwd("F:/ANAL/spatialCJS")

#read bird data
bird <- read.table("woth.txt", header=TRUE, sep=",", na.strings=c(NA,"NA","N","."))

#create spatial grid; in this version I use 1-degree cells 
xgrid <- GridTopology(c(-165.5,24.5), c(1,1), c(112, 48))
xdata <- as.data.frame(c(1:5376))
MAPSgrid <- SpatialGridDataFrame(xgrid, xdata)
writeAsciiGrid(MAPSgrid, "MAPSgrid.txt")

modelgrid=readAsciiGrid("MAPSgrid.txt")

#define station locations boundary with clickpoly   
dlocs <- bird[,18:17]
long=as.vector(dlocs$LONG)
lat=as.vector(dlocs$LAT)
plot(coordinates(modelgrid), pch = 20, cex = 0.5, col = "grey70")
points(dlocs,pch=20, cex=1)
bound <-clickpoly(add=TRUE) # just left click around the perimeter then right click when finished (ie to connect the last 2 pts).
 
#define spatial grid based on overlaying convex hull from point locations and model grid
tmp <- point.in.polygon(coordinates(modelgrid)[,1],coordinates(modelgrid)[,2], bound$bdry[[1]]$x, bound$bdry[[1]]$y)
polygon.mask <- tmp
modelgrid$MAPSgrid.txt<-polygon.mask
modelgrid$MAPSgrid.txt[modelgrid$MAPSgrid.txt == 0] <- NA
xp <- as(modelgrid, "SpatialPixelsDataFrame")
pp <- as.SpatialPolygons.SpatialPixels(xp)
sp2WB(pp, filename="modelgrid.txt")

#create neighbor list for each grid cell, default = 'queen's' neighborhood
neighbors <- poly2nb(pp) 

#subset station and grid data
data.subset<-point.in.polygon(dlocs[,1], dlocs[,2], bound$bdry[[1]]$x, bound$bdry[[1]]$y)
grid.subset<- coordinates(modelgrid)[!is.na(modelgrid$MAPSgrid.txt),]

#subset bird data based on stations in spatial grid
data<-bird[data.subset==1,]

y <- as.matrix(data[,5:16])
CAP <- as.vector(data[,"CAP"]) # covariate for p

# create indicator variable for pre-determined resident i.e., r(i) = 1
resident<-rep(0,nrow(y))
resident[data[,4]=="M"]<-1

# assign birds to grid cells
# spDistsN1 (from package "sp") creates distance vector (dvec) for each individual
# to store distances from stations (from "dlocs") to each grid point (from "grid.subset").  
# Grid id's are assigned based on the index of dvec corresponding to the grid point 
# for which the distance is smallest. 

gridid<-rep(NA,nrow(dlocs))
for(i in 1:nrow(dlocs) ){
  dvec <- spDistsN1(grid.subset,as.matrix(dlocs[i,],ncol=2) )
  gridid[i]<- (1:length(dvec))[dvec==min(dvec)]
}

# define no. individuals, no. yrs, and create empty vectors for first and last caps.
nind<-nrow(y)
nyear<-12
last<-first<-rep(NA,nind)

# set up starting values for "alive state" used in WinBUGS model specification.
Zst<-y

for(i in 1:nind){
h<-y[i,]
first[i]<- min( (1:12)[!is.na(h) & h==1])
last[i]<- max( (1:12)[!is.na(h)  & h==1])
Zst[i,first[i]:last[i]]<-1
Zst[i,1:first[i]]<-1
}
Zst[is.na(Zst)]<-0
Zst[Zst==0]<-1

# get rid of some unneeded data (initial capture in last year)
y<-y[first<12,]
lat<-lat[first<12]
long <- long[first<12]
dlocs<-cbind(long,lat)
Zst<-Zst[first<12,]
r<-resident[first<12]
last<-last[first<12]
gridid<-gridid[first<12]
CAP<-CAP[first<12]
first<-first[first<12]
nind<-nrow(y)
CAP<-CAP-mean(CAP) # center covariate

# objects needed for CAR model
num<-sapply(neighbors, length) # vector with elements = no. of neighbors of each grid cell.
adj<-unlist(neighbors)  # describes adjacency structure of grid.
sumNumNeigh<-length(unlist(neighbors)) #length of weights = no. adjacent pairs of grid cells.
ngrid<-length(num)

# write file with WinBUGS model specification
modelFilename = 'spatialcjs.txt'
cat('
model {
for(t in 1:nyear){
  p0[t] ~ dunif(0,1)
  lp0[t] <- log(p0[t]/(1-p0[t]))
  phi0[t] ~ dunif(0,1)
  lphi0[t] <- log(phi0[t]/(1-phi0[t]))
  rho0[t] ~ dunif(0,1)
  lrho0[t] <- log(rho0[t]/(1-rho0[t]))
  pi0[t] ~ dunif(0,1)
  lpi0[t] <- log(pi0[t]/(1-pi0[t]))
}

beta ~ dnorm(0,0.001)

for(j in 1:sumNumNeigh) { weights[j] <- 1}
  tau.phi ~ dgamma(.1,.1)
  sd.phi <- 1/pow(tau.phi, 0.5)
  tau.pi ~ dgamma(.1,.1)
  sd.pi <- 1/pow(tau.pi, 0.5)
  u[1:ngrid] ~ car.normal(adj[], weights[], num[], tau.phi)
  v[1:ngrid] ~ car.normal(adj[], weights[], num[], tau.pi)

for(i in 1:nind){
    for(j in 1:first[i]){
       z[i,j] ~ dbern(1)
     }
 logit(rho[i]) <- lrho0[first[i]+1]
 R[i] ~ dbern(pi[i,first[i]+1])
 mu[i] <- R[i]*rho[i]
 r[i] ~ dbern(mu[i])

for(t in (first[i]+1):nyear){
 logit(pi[i,t]) <- lpi0[t]+ v[gridid[i]]
 logit(pvec[i,t]) <- lp0[t] + beta*CAP[i]
 logit(phi[i,t]) <- lphi0[t] + u[gridid[i]]
        mu1[i,t] <- pvec[i,t]*z[i,t]
        y[i,t] ~ dbern(mu1[i,t])
        mu2[i,t] <- z[i,t-1]*phi[i,t]*R[i]
        z[i,t] ~ dbern(mu2[i,t])
}
}
}
', fill=TRUE, file=modelFilename)

# data input for WinBUGS
data <- list ("y", "first", "nind", "nyear", "r", "gridid", "num", "adj", "sumNumNeigh", "ngrid", "CAP")

# initial values for WinBUGS
ust<-as.vector(rnorm(ngrid,0,.4))
vst<-as.vector(rnorm(ngrid,0,.4))
Rst<-rep(1,nrow(y))
inits <- function(){
list (phi0=runif(nyear,.3,.6), rho0=runif(nyear,.3,.6), pi0=runif(nyear,.3,.6), p0=runif(nyear,.2,.6), beta=runif(1,-1,1), z=Zst, R=Rst, u=ust, v=vst)
}

#parameters to trace
parameters <- c("phi0", "p0", "beta", "rho0", "pi0", "tau.phi", "sd.phi", "tau.pi", "sd.pi", "u", "v")

#call WinBUGS, store results in 'out'
out <- bugs (data, inits, parameters, "spatialcjs.txt", n.thin=2, n.chains=2, n.burnin=10000, n.iter=30000, debug=FALSE)

########MAKE ASCII GRIDS FOR MAPPING PHI#######################################################

# average phi across years for each grid cell
lphi0 <- apply(out$sims.list$phi0, 2, logit)
lphi0 <- lphi0[,2:12] # get rid of phi0[1] which is not identifiable (is a non-interval)
lphi0.bar <- apply(lphi0, 1, mean)
lphi.grid <- lphi0.bar + out$sims.list$u[1:nrow(out$sims.list$u),]
phi.grid <- inv.logit(lphi.grid)               

#create grid for mean phi
mn.phi.grid <- apply(phi.grid, 2, mean)
mn.phi.grid <-cbind(grid.subset, mn.phi.grid)
mn.phi.grid <- as.data.frame(mn.phi.grid)
coordinates(mn.phi.grid)=c("s1","s2")
gridded(mn.phi.grid)<-TRUE
mn.phi.grid <- as(mn.phi.grid, "SpatialGridDataFrame")
write.asciigrid(mn.phi.grid, "F:/ANAL/SpatialCJS2/woth_mnphi_grid.asc") # write ascii grid file for import into arcgis

#create grid for sd phi
sd.phi.grid <- apply(phi.grid, 2, sd)
sd.phi.grid <-cbind(grid.subset, sd.phi.grid)
sd.phi.grid <- as.data.frame(sd.phi.grid)
coordinates(sd.phi.grid)=c("s1","s2")
gridded(sd.phi.grid)<-TRUE
sd.phi.grid <- as(sd.phi.grid, "SpatialGridDataFrame")
write.asciigrid(sd.phi.grid, "F:/ANAL/SpatialCJS2/woth_sdphi_grid.asc") # write grid file for import into arcgis

##########MAKE ASCII GRIDS FOR MAPPING PI ####################################################

# average pi across years for each grid cell
lpi0 <- apply(out$sims.list$pi0, 2, logit)
lpi0 <- lpi0[,2:12] # get rid of phi0[1] which is not identifiable (is a non-interval)
lpi0.bar <- apply(lpi0, 1, mean)
lpi.grid <- lpi0.bar + out$sims.list$v[1:nrow(out$sims.list$v),]
pi.grid <- inv.logit(lpi.grid)

#create grid for mean pi
mn.pi.grid <- apply(pi.grid, 2, mean)
mn.pi.grid <-cbind(grid.subset, mn.pi.grid)
mn.pi.grid <- as.data.frame(mn.pi.grid)
coordinates(mn.pi.grid)=c("s1","s2")
gridded(mn.pi.grid)<-TRUE
mn.pi.grid <- as(mn.pi.grid, "SpatialGridDataFrame")
write.asciigrid(mn.pi.grid, "F:/ANAL/SpatialCJS2/woth_mnpi_grid.asc")  # write grid file for import into arcgis

#create grid for sd phi
sd.pi.grid <- apply(pi.grid, 2, sd)
sd.pi.grid <-cbind(grid.subset, sd.pi.grid)
sd.pi.grid <- as.data.frame(sd.pi.grid)
coordinates(sd.pi.grid)=c("s1","s2")
gridded(sd.pi.grid)<-TRUE
sd.pi.grid <- as(sd.pi.grid, "SpatialGridDataFrame")
write.asciigrid(sd.pi.grid, "F:/ANAL/SpatialCJS2/woth_sdpi_grid.asc") # write grid file for import into arcgis

################################################################################

#calculate correlation between mean phi and mean pi
cor.vec <- rep(NA, nrow(phi.grid))
for (i in 1:nrow(phi.grid)){
tmp <- as.matrix(cbind(phi.grid[i,], pi.grid[i,]))
cor.vec[i] <- corr(tmp)
}



